(* This file belongs to the Secure Sessions Project 	      *)
(* http://www.msr-inria.inria.fr/projects/sec/sessions/       *)
(* MSR-INRIA Joint Centre, 2007    			      *)

open Conf
open Printf
open Prins

let _ = 
  let alice = {id="alice"; cert="alice"; ip="127.0.0.1"; port=8001} in
  let bob = {id="bob"; cert="bob.pub"; ip="127.0.0.1"; port=8002} in
  let charlie = {id="charlie"; cert="charlie"; ip="127.0.0.1"; port=8003} in
    register alice; register bob; register charlie

(* The reviewing process = 4 different incoming messages *)
let rec handler_response = 
{ hAccept = (fun _ comments -> 
      FinalVersion("Final", "Accepted! " ^ comments));
  hReject = (fun _ comments -> 
      "Rejected because " ^ comments);
  hShepherd = (fun _ questions -> 
      (* Very simple answering strategy *)
      Rebuttal("Let me in!", handler_response));
  hRevise = (fun _ reviews -> 
      Submit("Paper", handler_response)) }

(* Exchange with the confman about formatting *)
let rec handler_format = 
{ hBadFormat = (fun _ error -> 
      printf "Formatting error: %s\n" error;
      Upload("Submission", handler_format)); 
  hOk = (fun _ s -> 
	     Submit("Submission", handler_response)) }

(* First message is always a Cfp *)
let handler_cfp =
{ hCfp = fun p s -> 
      Upload("First draft", handler_format)}

let _ =
  let result = author "alice" handler_cfp in
  printf "Author session complete: %s\n" result








