(* This file belongs to the Secure Sessions Project 	      *)
(* http://www.msr-inria.inria.fr/projects/sec/sessions/       *)
(* MSR-INRIA Joint Centre, 2007    			      *)

open Conf
open Printf
open Prins

let _ = 
  let alice = {id="alice"; cert="alice"; ip="127.0.0.1"; port=8001} in
  let bob = {id="bob"; cert="bob"; ip="127.0.0.1"; port=8002} in
  let charlie = {id="charlie"; cert="charlie"; ip="127.0.0.1"; port=8003} in
    register alice; register bob; register charlie

let start_confman () =
  let rec handler_decision = { 
      hClose = (fun _ _ -> Done((), "No more revisions")); 
      hReqRevision = (fun _ r -> Revise(r, handler_submission))
    }
  and handler_submission = { 
      hSubmit = (fun _ submit -> Paper(submit, handler_decision)); 
      hWithdraw = (fun _ _ -> Retract((), "Retracted"))
    } in
  let rec handler_paper counter prins draft = 
    if counter > 0 
    then BadFormat("Busy now."^(string_of_int counter),
                  {hUpload = handler_paper (counter-1)}) 
    else 
      if String.length draft > 12 
    then BadFormat("Make it shorter!",{hUpload = handler_paper 0}) 
    else Ok((),handler_submission) in
  let result = confman "bob" {hUpload = handler_paper 10} in
  printf "ConfMan: session complete: %s\n\n" result

let start_pc () =
  let (prins:principals) = 
    { pc = "charlie";
      author = "alice"; 
      confman = "bob" } in
  let cfp = "Call for papers" in
  let rec handler_discuss counter = {
      hRebuttal = (fun _ _ -> 
        if counter >0 then
          Shepherd("Do you really want to be in?", handler_discuss (counter-1))
        else Accept("Ok then ...", {
	    hFinalVersion = (fun _ s -> "We accepted the following paper: "^s)
          }))
    } in
  let rec handler_paper counter = { 
      hPaper = (fun _ s -> 
	if counter > 0 
        then ReqRevision("Busy."^(string_of_int counter), 
                        handler_paper (counter-1))
        else
	  if s.[0] = 'S' 
	then ReqRevision("Make it better!", handler_paper 0) 
	else Close((),{
	    hDone = (fun _ _ -> Shepherd("Do you really want to be in?", 
                                        handler_discuss 10))
	  }));
      hRetract = (fun _ _ -> "Retracted") } in
  let result = pc prins (Cfp(cfp, handler_paper 10)) in
  printf "PC: session complete: %s\n\n" result

let start_author () =
  (* The reviewing process = 4 different incoming messages *)
  let rec handler_response = 
    { hAccept = (fun _ comments -> 
      FinalVersion("Final", "Accepted! " ^ comments));
      hReject = (fun _ comments -> 
        "Rejected because " ^ comments);
      hShepherd = (fun _ questions -> 
        (* Very simple answering strategy *)
        Rebuttal("Let me in!", handler_response));
      hRevise = (fun _ reviews -> 
        Submit("Paper", handler_response)) } in
  (* Exchange with the confman about formatting *)
  let rec handler_format = 
    { hBadFormat = (fun _ error -> 
      printf "Formatting error: %s\n" error;
      Upload("Submission", handler_format)); 
      hOk = (fun _ s -> 
        if true (* We could flip a coin *)
        then Submit("Submission", handler_response) 
        else Withdraw((), "Paper withdrawn")) } in
  (* First message is always a Cfp *)
  let handler_cfp =
    { hCfp = fun p s -> 
      Upload("First draft", handler_format)} in
  let result = author "alice" handler_cfp in
  printf "Author session complete: %s\n" result

let _ = Pi.fork start_confman 
let _ = Pi.fork start_author
let _ = start_pc ()
    
