(* This file belongs to the Secure Sessions Project 	      *)
(* http://www.msr-inria.inria.fr/projects/sec/sessions/       *)
(* MSR-INRIA Joint Centre, 2007    			      *)

open Conf
open Printf
open Prins

let _ = 
  let alice = {id="alice"; cert="alice"; ip="127.0.0.1"; port=8001} in
  let bob = {id="bob"; cert="bob.pub"; ip="127.0.0.1"; port=8002} in
  let charlie = {id="charlie"; cert="charlie"; ip="127.0.0.1"; port=8003} in
    register alice; register bob; register charlie

let (prins:principals) = 
  { pc = "charlie";
    author = "alice"; 
    confman = "bob" }

let cfp = "Call for papers"

let rec handler_discuss counter = {
  hRebuttal = (fun _ _ -> 
    if counter >0 then
      Shepherd("Do you really want to be in?", handler_discuss (counter-1))
    else Accept("Ok then ...", {
		  hFinalVersion = (fun _ s -> "We accepted the following paper: "^s)
      }))
  }


let rec handler_paper counter = { 
    hPaper = (fun _ s -> 
	if counter > 0 then ReqRevision("Busy."^(string_of_int counter), handler_paper (counter-1)) else
	  if s.[0] = 'S' 
	  then ReqRevision("Make it better!", handler_paper 0) 
	  else Close((),{
		       hDone = (fun _ _ -> Shepherd("Do you really want to be in?", handler_discuss 10))
		     }));
  hRetract = (fun _ _ -> "Retracted") }
  

let _ = 
  let result = pc prins (Cfp(cfp, handler_paper 10)) in
  printf "PC: session complete: %s\n\n" result
 
