(* This file belongs to the Secure Sessions Project 	      *)
(* http://www.msr-inria.inria.fr/projects/sec/sessions/       *)
(* MSR-INRIA Joint Centre, 2007    			      *)

open S3
open Printf

(* Gui *)

let window = GWindow.window ~border_width:10 ~height:200 ~width:300 ()

let box = GPack.box `VERTICAL ~packing:window#add () 

let label = GMisc.label ~text:"Session not running" ~packing:box#add ()

let entry = GEdit.entry ~show:false ~packing:box#add ()

let button_start = GButton.button ~label:"Start Session" ~packing:box#add ()

let button_accept = GButton.button ~label:"Accept" ~packing:box#add ~show:false ()
let button_change = GButton.button ~label:"Change" ~packing:box#add ~show:false ()

let button_quit = GButton.button ~label:"Quit" ~packing:box#add ~show:false ()


let button_end = GButton.button ~label:"Next" ~packing:box#add ~show:false ()

(* Actions *)
let delete_event ev =
(*  flush_all ();*)
  GMain.Main.quit () ;
  true

let destroy_window () = GMain.Main.quit ()

(* Session *)

let _ = Prins.register "alice" "127.0.0.1" 8001
let _ = Prins.register "bob" "127.0.0.1" 8002
let _ = Prins.register "charlie" "127.0.0.1" 8003

let (prins:principals) = { customer = "alice"; 
                           officer = "charlie";
                           store = "bob"; }

type answer =
    | Accepting
    | Changement of string
    | Undecided

let v = ref Undecided

let rec handler_offer = 
  { hReject = (fun _ t -> begin 
      label#set_text t;
      button_accept#misc#hide ();
      button_change#misc#hide ();
      entry#misc#hide ();
      button_quit#misc#show ();
      entry#misc#hide ();
      Abort((),())
    end) ;
    hOffer = (fun _ offer -> 
      label#set_text ("Offer received: "^offer);
      button_start#misc#hide ();
      button_accept#misc#show ();
      button_change#misc#show ();
      entry#misc#show ();
      while !v = Undecided do Thread.yield() done;
      match !v with
        | Accepting -> begin
              label#set_text "Session completed";
              button_accept#misc#hide ();
              button_change#misc#hide ();
              entry#misc#hide ();
              button_quit#misc#show ();
              Accept((),())
          end
        | Changement c -> begin
              v:= Undecided ;
              Change(c, handler_offer)
          end)}
    
let accept () =
  v:= Accepting

let change () =
  v:= Changement entry#text

let start_session () = 
  label#set_text "Session started";
  customer prins (Request("12 March 2007",handler_offer));
  printf "Customer: session complete.\n\n"

let start () = ignore (Thread.create start_session ())

let init () =
  window#event#connect#delete ~callback:delete_event;
  window#connect#destroy ~callback:destroy_window;
  button_start#connect#clicked ~callback:start;
  button_accept#connect#clicked ~callback:accept;
  button_change#connect#clicked ~callback:change;
  button_quit#connect#clicked ~callback:destroy_window;
  window#show ();
  GtkThread.main ()  

let _ = init ()
