(* This file belongs to the Secure Sessions Project 	      *)
(* http://www.msr-inria.inria.fr/projects/sec/sessions/       *)
(* MSR-INRIA Joint Centre, 2007    			      *)

open System.Windows.Forms
open System.Threading
open Printf
open S3

#nowarn "40";; // for value recursion

type ChoiceOnOffer = UChange of string | UAccept

type GUI0 = interface // GUI as wired in C# form
  abstract GetReady: unit -> AutoResetEvent
  abstract Offer_event: string -> unit
  abstract Offer_result: unit -> ChoiceOnOffer
  abstract Reject_event: unit -> unit
end
    
type GUI1 = interface // GUI as functions used in F#
  abstract Offer_ui: string -> ChoiceOnOffer
  abstract Reject_ui: unit -> unit
end

// We call GUI code only from its Form thread
let remoteInvoke (form: #Form) f = 
  form.BeginInvoke(new MethodInvoker(f)) |> ignore

// We then wait for something to happen
let wait (x : AutoResetEvent) = 
  WaitHandle.WaitAny([| (x:> WaitHandle) |]) |> ignore;
  if x.Reset() then () else failwith "cannot reset wait"

let wrapGUI (gui: GUI0) (form : Form)  = 
   { new GUI1 with 
      member self.Offer_ui v =
         remoteInvoke form (fun () -> gui.Offer_event(v));
         wait (gui.GetReady());
         gui.Offer_result() 
      member self.Reject_ui () = 
         remoteInvoke form (fun () -> gui.Reject_event())
     end }

let Start_customer gui form prins r = 
  let gui = wrapGUI gui form in
  let rec msg1 = { 
    hReject = (fun _ _ -> 
      gui.Reject_ui(); Abort((),()));
    hOffer = (fun _ offer -> 
      match gui.Offer_ui offer with
      | UChange location -> Change(location,msg1)
      | UAccept -> Accept((),()))} in
  let msg0 = Request(r,msg1) in
  let worker () = 
	customer prins msg0;
	printf "Worker thread completed." in
  (new Thread(new ThreadStart(worker))).Start()
