(* This file belongs to the Secure Sessions Project 	      *)
(* http://www.msr-inria.inria.fr/projects/sec/sessions/       *)
(* MSR-INRIA Joint Centre, 2007    			      *)

open Unix
open Prins

let _ = 
  let alice = {id="alice"; cert="alice"; ip="127.0.0.1"; port=8001} in
  let bob = {id="bob"; cert="bob.pub"; ip="127.0.0.1"; port=8002} in
  let charlie = {id="charlie"; cert="charlie"; ip="127.0.0.1"; port=8003} in
  register alice; register bob; register charlie

let victims = [ ("alice",("127.0.0.1",8001)) ; 
                ("bob",("127.0.0.1",8002)) ; 
                ("charlie",("127.0.0.1",8003)) ]

let rec showmsgs msglist n = 
  match msglist with
    | [] -> ""
    | a::s -> (string_of_int n)^". "^a^"\n"^showmsgs s (n+1) 

let send_msg ml d = 
  Printf.printf "Where to send msg %d?\n" d;
  let inp = read_line () in  
  let udp = socket PF_INET SOCK_DGRAM 0 in
  let (url,destport) = List.assoc inp victims in
  let addr = inet_addr_of_string url in
  let text = List.nth ml d in
    sendto udp text 0 (String.length text) [] (ADDR_INET(addr,destport))

let _ =
  let udp = Unix.socket Unix.PF_INET Unix.SOCK_DGRAM 0 in
   Unix.bind udp (Unix.ADDR_INET (Unix.inet_addr_any, 8666));
    let msglist = ref [] in
   while true do
     Printf.printf "waiting for messages at 8666...\n"; flush_all();
     let text = String.create 1024 in
     let (len,_) = recvfrom udp text 0 1024 [] in
       Printf.printf "intercepted:\n%s\n" text; 
        msglist := (String.sub text 0 len)::!msglist;
	 Printf.printf "Eavesdropped messages:\n%s\nChoose a message." (showmsgs !msglist 0); flush_all ();
	 let inp = read_line () in  
	   (send_msg !msglist (int_of_string inp));
   done
