(* This file belongs to the Secure Sessions Project 	      *)
(* http://www.msr-inria.inria.fr/projects/sec/sessions/       *)
(* MSR-INRIA Joint Centre, 2007    			      *)

module Prins

open Pi
open Data
open Base64
open Crypto
open System.Net

open System.Security.Cryptography.X509Certificates
open System.Security.Cryptography

type pr = { id:string; cert:string; ip:string; port:int}

let addrDb : (string * (string * int)) list ref = ref []
let localaddrDb : (string * (string * int)) list ref = ref []


let register (prin:pr) =
  addrDb := (prin.id,(prin.ip,prin.port))::!addrDb ;
  localaddrDb := (prin.id,(prin.ip,prin.port))::!localaddrDb

(* let register (name:string) (inet:string) (port:int) = *)
(*   addrDb := (name,(inet,port))::!addrDb ; *)
(*   localaddrDb := (name,(inet,port))::!localaddrDb *)


let check_cache (prin:string) (role:int) (sid:bytes) = 
()

let localMachine:X509Store = 
  new X509Store(StoreName.My,StoreLocation.LocalMachine) 
let currentUser:X509Store = 
  new X509Store(StoreName.My,StoreLocation.LocalMachine) 
exception Cert_Not_Found of string

let getPrinFromStore (store:X509Store) (subject:string)  =
  store.Open(OpenFlags.ReadOnly);
  let certs = store.get_Certificates() in
  let search = certs.Find(X509FindType.FindBySubjectName,subject,false) in 
  if (search.get_Count()) < 1 then raise (Cert_Not_Found subject )
  else  
    let cert = search.Item(0) in
      if (cert.get_HasPrivateKey()) = false then raise (Cert_Not_Found ("No priv key:"^subject) )
      else  
	let priv = (cert.get_PrivateKey()).ToXmlString(true) in
	  (subject, (cert.Export(X509ContentType.Cert)),
	    (asympub priv),
	    (asympriv priv))

let showStore ()= 
  let printCert (c:X509Certificate2) = 
    let str = "SUBJECT: "^c.Subject^" - ISSUER: "^c.Issuer^" - Privatekey: "^(if c.HasPrivateKey then "yes" else "no") in 
(*    let str = c.ToString() in*)
    (Printf.printf "%s\n" str) in
  let (mach:X509Store) = new X509Store(StoreName.My,StoreLocation.LocalMachine) in
    mach.Open(OpenFlags.ReadOnly);
    let certs = mach.get_Certificates() in
    let n = certs.get_Count()-1 in
      Printf.printf "\nCurrent Certificate Store\n";
      Printf.printf "*************************\n";
      for i=0 to n do
	(printCert (certs.Item(i))) done



let privkey = ibase64 (cS
"MIICXQIBAAKBgQC+L6aB9x928noY4+0QBsXnxkQE4quJl7c3PUPdVu7k9A02hRG4
81XIfWhrDY5i7OEB7KGW7qFJotLLeMec/UkKUwCgv3VvJrs2nE9xO3SSWIdNzADu
kYh+Cxt+FUU6tUkDeqg7dqwivOXhuOTRyOI3HqbWTbumaLdc8jufz2LhaQIDAQAB
AoGAIbKXshdzP8Qe1iIsctaAYlzC2IrBEhQLpoH4cFNi6LZFUQ+q4DZdULTHt5Aj
VmvaQlkHGJMXiNCMwPiZhbtrIVb6EaC+3leFQntQH9989uBOOvnGOYK8TYR7fy8g
p2SnIrbwnK81xMVHOZ+ip4F9JjCoJFu7JGQ8LJys4clFfMECQQDdm0DHgEkFTqnX
6gXRYAvzT5ymNnLg4uoeufUDdZfpIF98sHWeyin7SaNRvZeOUjfWKZMhHfcxDujz
BWrnyb2VAkEA27QFsZNIvAU91EzVgk//W1EETj3BmWoMMJ0RQcl+4KZn5H9SJiL2
1lV0qV+vPyI240pE8NZ4/2I9TcBWdJ8XhQJBAMz4UaV1QZT4Gl5L5c+wH72PsFuI
Vx8sotGCvF3TwYtuVwOUj2YspUH47Weojeqkn2KmF6D0/3NP0CUfqK71STUCQQDY
UTzV5Y1UythgwrHqF3f2YiOaIcYPllGgP9Xw70LEUDO00AODVyOwTv2JkpOK1tYy
HjsY4iLKWODBQCIfu1C9AkBS+s04OsH3jJRGc74KsAhqC7r056rb8oOOc+6X3K4N
nB3YAlyMvZBK3I8BorugTnrxANak1kZPm79LZJ1r3M/7")

let cert = ibase64 (cS
"MIIBxDCCAW6gAwIBAgIQxUSXFzWJYYtOZnmmuOMKkjANBgkqhkiG9w0BAQQFADAW
MRQwEgYDVQQDEwtSb290IEFnZW5jeTAeFw0wMzA3MDgxODQ3NTlaFw0zOTEyMzEy
MzU5NTlaMB8xHTAbBgNVBAMTFFdTRTJRdWlja1N0YXJ0Q2xpZW50MIGfMA0GCSqG
SIb3DQEBAQUAA4GNADCBiQKBgQC+L6aB9x928noY4+0QBsXnxkQE4quJl7c3PUPd
Vu7k9A02hRG481XIfWhrDY5i7OEB7KGW7qFJotLLeMec/UkKUwCgv3VvJrs2nE9x
O3SSWIdNzADukYh+Cxt+FUU6tUkDeqg7dqwivOXhuOTRyOI3HqbWTbumaLdc8juf
z2LhaQIDAQABo0swSTBHBgNVHQEEQDA+gBAS5AktBh0dTwCNYSHcFmRjoRgwFjEU
MBIGA1UEAxMLUm9vdCBBZ2VuY3mCEAY3bACqAGSKEc+41KpcNfQwDQYJKoZIhvcN
AQEEBQADQQAfIbnMPVYkNNfX1tG1F+qfLhHwJdfDUZuPyRPucWF5qkh6sSdWVBY5
sT/txBnVJGziyO8DPYdu2fPMER8ajJfl")

let get_pubkey (subj:str) =
(*  let (subject, cert, pubk, privk) = getPrinFromStore localMachine subj in
    pubk  *)
  let (s,a,p) = ix509 cert in asympubkey p

	   
let get_privkey (subj:str) =
(*  showStore ();
  let (subject, cert, pubk, privk) = getPrinFromStore localMachine subj in 
    privk *)
  asymprivkey privkey

(* for now ignored, using global address_table  *)
let address (subj:str) = ()

let masterSecret = cS "toto"
let get_sharedkey (prin:str) (sh:str) = sha1 (concat3 (utf8 prin) (utf8 sh) (utf8 masterSecret) )

(* insider attacks *)

let leakEv = Pi.trace()

let leak_privkey (u:str) = 
  let privk = get_privkey u in
    Pi.log leakEv u;
    privk

let leak_sharedkey (u:str) (p:str) = 
  let sharedk = get_sharedkey u p in
    Pi.log leakEv (u^p);
    sharedk 

(* x509Store functions *) 
let add (subj:str) = 
 let (mach:X509Store) = new X509Store(StoreName.My,StoreLocation.LocalMachine) in
   mach.Open(OpenFlags.ReadWrite);
   let newCert = new X509Certificate2(subj^".cer") in
     mach.Add(newCert); mach.Close()
	   
  
let rem (subj:str) = 
 let (mach:X509Store) = new X509Store(StoreName.My,StoreLocation.LocalMachine) in
   mach.Open(OpenFlags.ReadWrite);
   let certs = mach.get_Certificates() in
   let certs2 = certs.Find(X509FindType.FindBySubjectName,subj,false) in
     mach.RemoveRange(certs2); mach.Close()


let get_address a subject = 
  List.assoc subject a

let udp = ref (stderr)

let bind prin = ()

let close () = ()

(* preliminary steps done elsewhere: 

C:\Program Files\Microsoft Visual Studio 8\SDK\v2.0>Makecert -n "CN=client" client.cer
Succeeded

C:\Program Files\Microsoft Visual Studio 8\SDK\v2.0>Makecert -n "CN=server" server.cer
Succeeded

C:\Program Files\Microsoft Visual Studio 8\SDK\v2.0>cp client.cer "c:\Documents and Settings\t-ricarc\inria-msr-sec\lang-sec\session-types\src\crypto"

C:\Program Files\Microsoft Visual Studio 8\SDK\v2.0>cp server.cer "c:\Documents and Settings\t-ricarc\inria-msr-sec\lang-sec\session-types\src\crypto"
*)


let rec find_destination label role_info = 
  let rec isin label list = match list with [] ->  false | a::l -> label=a || isin label l in
  let rec find label count role_info =
  match role_info with
    | [] -> assert false
    | l::r -> if isin label l then count else find label (count+1) r in
    find label 0 role_info


let psend  (subj:string) (text:string): unit = 
  let (url, port) =  List.assoc subj !addrDb in
  let udp = new Sockets.UdpClient(port+50) in
  let m = utf8 text in
 (* let cl = "Content-Length:"^(string_of_int (length m)) in *)
  let msg = utf8(text) in
  (*  Pi.yield 1000; *)
    udp.Connect (url, port); 
    udp.Send (msg, msg.Length); 
    udp.Close(); 
    Printf.printf  "\n%s sent: %s\n\n" subj text


let precv (self:string) : string =
  let (url, port) =  List.assoc self !addrDb in
  let ipendpoint = new System.Net.IPEndPoint(IPAddress.Any, 
					 0) in
   let udp = new Sockets.UdpClient(port) in
(*     Printf.printf  "Listening:\n port   : %d\n" port; *)
     let s = udp.Receive(ref ipendpoint) in
     let recv = iutf8 s in
       (* Printf.printf  "received!. Processing...\n"; *)
       udp.Close ();
(*     let len = 
       if s.StartsWith "Content-Length:" then
	 let i = String.length "Content-Length:" in
	 let s = s.Substring (i,s.Length - i) in
	   Printf.printf "length str:%s\n" s;
	   int_of_string  s
       else failwith "no Content-Length found" in
       Printf.printf  "Message recd: Length:%d\n" len;
       let buf = Array.make len ' ' in  
       let len2 = sr.ReadBlock (buf, 0, len) in
	 assert (len = len2);
	 let s = new System.String(buf) in
	   // Printf.printf "\ns= %s\n" s; *)
	 recv


