open Unix
open Global
open Data
(*
external openssl_base64: string -> string -> unit = "base64" 
external openssl_ibase64: string -> string -> unit = "ibase64"
*)
external openssl_mkNonce: unit -> string = "mkNonce" "noalloc"
external openssl_sha1: string -> string -> unit = "sha1" "noalloc"
external openssl_rsa_sha1: string -> string -> string -> unit = "rsa_sha1" "noalloc"
external openssl_rsa_verify: string -> string -> string -> bool = "rsa_verify" 

(* cryptography *)
let mkNonce () = if !crypto_on then utf8 (cS (openssl_mkNonce ())) else utf8 (cS "")

let sha1 msg = 
  if !crypto_on then
    let s = String.create 20 in
    openssl_sha1 (iS (iutf8 msg)) s ; utf8 (cS s)
  else
    utf8 (cS "")

let rsa_sign sk msg = 
  if !signing then
    let s = sha1 msg in
    let sign = String.create 128 in
    openssl_rsa_sha1 (iS (iutf8 sk)) (iS (iutf8 s)) sign;
    utf8 (cS sign)
  else 
    utf8 (cS "")

let rsa_verify pk msg s = 
  if !verifying then
  let m = (sha1 msg) in
    if openssl_rsa_verify (iS (iutf8 pk)) (iS (iutf8 m)) (iS (iutf8 s))
    then 
    ()
    else failwith ("RSA_verify failed: "^pk)
  else ()

(*let base64 s =
  let out = String.create 3048 in
  openssl_base64 s out;
    String.sub out 0 ((String.length s)*2)

let ibase64 s = 
let out = String.create 3048 in
   openssl_ibase64 s out; out
*)

(* testing: *)
(*let _ = let b = (base64 "YOYOasdcaskjnsalkfjrreanwlfdecmewrlkejrfn;qwelkdmq'23dkjwernfdwlekjdnwelkdjx$#R@$Ra990(u7y~?#@WSS@?WS?!#@R%$?R%$T@$~#@?@!#W") in Printf.printf "base64 =%s (%d)\n%s\n" b (String.length b) (ibase64 b)*)
(*let _ = let plaintext = "AFEWRDFW#@$R$r53o83u4j90r8324jroirfun2oi4uj8-4-98uj43-9rdjudpewoijd2-[3498dj34dpiu34djpde23';e.23';e1mklfd;m34pofij34p0-mkof9';wcmklf '[qwioerpfmk]24pofik43e2e23le,23l;ek2,3';e2kl3me,2;lk3em2;3lke12w12w;lkw1,2';3l,e21'12'wl;,#@$#R23211w21we12w21w21" in 
  Printf.printf "sent (%d):\n%s\nrecv (%d)\n%s\n" (String.length plaintext) (base64 plaintext) (String.length (ibase64 (base64 plaintext))) (ibase64 (base64 plaintext))*)
(*
let _ = Printf.printf "SHA1 =\n%s\n" (sha1 "YOYO")
let _ = Printf.printf "SHA1 =\n%s\n" (sha1 "asdmksdalkjcfnclwjehnlwekjdnlwieunYOYO")
let _ = Printf.printf "SHA1 =\n%s\n" (sha1 "asdmksdalkjcfnclwjehnlwekjdnlwieunYOYO")
*)

(*
let _ = Printf.printf "base64 =\n%s\n" (base64 "YaaaaaaaaaaaaaaaaaaaaaaaaaaOYnmn")
let _ = Printf.printf "ibase64 =\n%s\n" (ibase64 (base64 "YaaaaaaaaaaaaaaaaaaaaaaaaaaOYnmn"))


let _ = 
  let signature =  rsa_sign "test.key" "YOKK1asdfasdfsdfrneolrjkfnwerpifunerwf34rfYOKK1asdfasdfsdfrneolrjkfnwerpifunerwf34rfYOKK1asdfasdfsdfrneolrjkfnwerpifunerwf34rfYOKK1asdfasdfsdfrneolrjkfnwerpifunerwf34rf" in
    (rsa_verify "test.pub" "YOKK1asdfasdfsdfrneolrjkfnwerpifunerwf34rfYOKK1asdfasdfsdfrneolrjkfnwerpifunerwf34rfYOKK1asdfasdfsdfrneolrjkfnwerpifunerwf34rfYOKK1asdfasdfsdfrneolrjkfnwerpifunerwf34rf" signature)
*)
(*
let _ = let bs = rsa_sign "test.key" "hola" in rsa_verify "test.pub" bs "hola"
*)
