open Unix
open Global
open Data

  (* open Route *)
(* Routing tables *)
let addrDb : (string * (string * int)) list ref = ref []
let localaddrDb : (string * (string * int)) list ref = ref []

type pr = {id:string; cert:string; ip:string; port:int}

let register (prin:pr) =
  addrDb := (prin.id,(prin.ip,prin.port))::!addrDb ;
  localaddrDb := (prin.id,(prin.ip,prin.port))::!localaddrDb
let udp = ref (Unix.stderr)

(* communication *)
let psend (dest:string) (text:str): unit = 
  let (url,destport) = List.assoc dest !addrDb in
  let text = iS text in
  let udpSend = socket PF_INET SOCK_DGRAM 0 in
  let addr = inet_addr_of_string url in
  let _ = sendto udpSend text 0 (String.length text) [] (ADDR_INET(addr,destport)) in 
  let _ = close udpSend in
  debug "Prins" ("sent:\n"^text) ;
  ()
      
let precv (self:string) : str =
   let text = String.create 1024 in
   (*   let (len,_) = recvfrom udp text 0 1024 [] in *)
   let len = recv !udp text 0 1024 [] in
   debug "Prins" ("received:\n"^text) ;
   cS (String.sub text 0 len)

(* principal library *)

let get_privkey s = utf8 (cS ((iS s)^".key"))
let get_pubkey s = utf8 (cS ((iS s)^".pub"))

let get_address ad subject = 
  List.assoc subject ad

let bind prin =
  let (_,localport) = get_address (!localaddrDb) prin in
    udp := Unix.socket Unix.PF_INET Unix.SOCK_DGRAM 0;
    Unix.bind !udp (Unix.ADDR_INET (Unix.inet_addr_any, localport))

let close () = Unix.close !udp

(* Cache handling *)

let check_cache (prin:string) (role:int) (sid:bytes) = 
  let rec read_lines chan =
    let s = 
      try Some (input_line chan) with
          End_of_file -> None in
    match s with
        Some m -> (m)::(read_lines chan)
      | None -> [] in 
  let f = Unix.openfile (prin^(string_of_int role)^".cache") [Unix.O_RDWR; Unix.O_CREAT] 0o640 in
  let ch = Unix.in_channel_of_descr f in
  let l = read_lines ch in
  if List.mem (spstr (base64 sid)) l 
  then (
      Unix.close f;
      failwith "cache error" )
  else (
    debug "Prins" ("Adding "^(spstr (base64 sid))) ;
    let sid_string = (spstr (base64 sid))^"\n" in
    let _ = Unix.write f sid_string 0 (String.length sid_string) in
    Unix.close f)
