(* This file belongs to the Secure Sessions Project 	      *)
(* http://www.msr-inria.inria.fr/projects/sec/sessions/       *)
(* MSR-INRIA Joint Centre, 2007    			      *)


open Global
open Data

let _ = Random.init (int_of_float (Unix.time ()))

let mkNonce () = 
  let s = Printf.sprintf "%i" (Random.int (1000000)) in
  Bin (Nonce (utf8 (cS s)))

let sha1 s =
  Bin (Hash s)

let rsa_sign k t =
  Bin (AsymSign (k,t))

let test_eq k1 k2 = 
  let k1 = iS (iutf8 k1) in
  let k2 = iS (iutf8 k2) in
  let kk1 = String.sub k1 0 (String.length k1 -3) in
  let kk2 = String.sub k1 0 (String.length k2 -3) in
  kk1 = kk2


let rsa_verify k m s = 
  (*  debug "Crypto" ("Verifying: "^(spbytes s)^"\nagainst "^(spbytes m)^" with "^(spbytes k)^"\n");  *)
  match s with
      Bin (AsymSign (k',m')) -> assert (test_eq k k' && m = m')
    | _ -> assert false
