(* This file belongs to the Secure Sessions Project 	      *)
(* http://www.msr-inria.inria.fr/projects/sec/sessions/       *)
(* MSR-INRIA Joint Centre, 2007    			      *)

open Printf

type str =
    | Literal of string
    | Base64 of bytes
        
and bytes = 
    | Concat of bytes * bytes
    | Utf8 of str
    | Bin of blob

and blob = 
    | Hash of bytes
    | Nonce of bytes
(*    | DerivedKey of bytes * bytes     PSHA-1(basekey,nonce)/(utf8 pwd,seed) *)
    | AsymSign of bytes * bytes
    | AsymEncrypt of bytes * bytes

let rec spstr s = match s with
  | Base64(t) -> sprintf "%s" (spbytes t)
  | Literal(s) -> sprintf "%s" s
and spblob b = 
  let keyed name key arg = sprintf "%s{%s}[%s]" name (spbytes key) (spbytes arg) in
  match b with
    | AsymSign(k,t) -> keyed "RSA-SHA1" k t
    | AsymEncrypt(k,t) -> keyed "RSA-Enc" k t
    | Hash(s) -> sprintf "SHA1(%s)" (spbytes s)
    | Nonce(s) -> sprintf "Nonce(%s)" (spbytes s)
and spbytes b = 
  match b with
    | Utf8(s) -> sprintf  "\'%s\'" (spstr s)
    | Concat(x,y) -> sprintf "%s | %s" (spbytes x) (spbytes y)
    | Bin(b) -> sprintf "%s" (spblob b)

let cS s = Literal s
let iS s = match s with
  | Literal s -> s
  | _ -> failwith "iS failed"

let base64 b = Base64(b)
let ibase64 b = match b with
| Base64 s -> s
| _ -> failwith "ibase64 failed"

let append x y = Concat(x,y)
  
let concat x y = Concat(x,y)
let iconcat s = match s with
  | Concat(x,y) -> (x,y)
  | _ -> failwith "iconcat failed"
let concat3 x y z = concat x (concat y z)
let iconcat3 e = (let x,yz = iconcat e in let y,z = iconcat yz in x,y,z)

let utf8 x = Utf8 x
let iutf8 b = match b with
| Utf8 s -> s
| _ -> failwith "iutf8 failed"
