(* This file belongs to the Secure Sessions Project 	      *)
(* http://www.msr-inria.inria.fr/projects/sec/sessions/       *)
(* MSR-INRIA Joint Centre, 2007    			      *)

(********************************************************************)
(* Session - Implementation                                         *)
(*                                                                  *)
(* common.ml                                                        *)
(********************************************************************)
(* $Id: common.ml 2847 2007-06-25 14:40:15Z denielou $ *)

(* Debug functions *)

let debug_bool = true
let debug_p = false
let debug_g = false
let debug_graph = false

let debug d =
  if d then
    (fun modul msg -> begin
      print_string ("["^modul^"] ");
      print_string (msg);
      print_newline ();
      flush stdout
    end)
  else fun _ _ -> ()

let debug_common = debug debug_bool "Common"

(* Regression tests *)

let regression1 d s f x y = 
  let z = f x in
  if z = y 
  then d ("Regression "^s^" passed!\n")
  else (d ("Regression "^s^" failed!\n");exit 1)

let regression2 d s f x0 x1 y = 
  let z = f x0 x1 in
  if z = y 
  then d ("Regression "^s^" passed!\n")
  else (d ("Regression "^s^" failed!\n");exit 1)

let regression3 d s f x0 x1 x2 y = 
  let z = f x0 x1 x2 in
  if z = y 
  then d ("Regression "^s^" passed!\n")
  else (d ("Regression "^s^" failed!\n");exit 1)

(* Positionning *)

type pos = int * int
type info = pos * pos

let bogusInfo = ((0,0),(0,0))

let info_to_string ((l1,c1),(l2,c2)) =
  let s = Printf.sprintf " line %d, char %d, to line %d, char %d " l1 c1 l2 c2 in
(*  debug_common s ; *)
  s

let merge_info (a,_) (_,d) = (a,d)

(* String parsing *)

let split s =
  if String.contains s '.'
  then
    let n = String.length s in
    let i = String.index s '.' in
    (String.sub s 0 i,String.sub s (i+1) (n-1-i))
  else
    ("",s)


(* Errors *)
exception Syntax_error of string*info
exception Parse_error of string*info

