(* This file belongs to the Secure Sessions Project 	      *)
(* http://www.msr-inria.inria.fr/projects/sec/sessions/       *)
(* MSR-INRIA Joint Centre, 2007    			      *)

(********************************************************************)
(* Session - Implementation                                         *)
(*                                                                  *)
(* main.ml                                                          *)
(********************************************************************)
(* Time-stamp: <2006-07-25 15:08:26 t-piermd> *)

open Syntax

let debug = Common.debug Common.debug_bool "main"

(* Main function *)
let compile_session session =
  debug "Compilation starting" ;
  let lexbuf = Lexing.from_string session in
  debug "Building lexer" ;
  let (session_name,desc,_ (*code*) ) =
    (try 
        Parser.sess Lexer.token lexbuf
      with
          Common.Syntax_error (s,i) ->
	    (debug ("Syntax error: "^s^(Common.info_to_string i)); flush_all () ; exit 1)
        | Common.Parse_error (s,i)  ->
	    (debug ("Parsing error: "^s^(Common.info_to_string i)); flush_all () ; exit 2)
    ) in
  if Common.debug_p then
    List.iter 
      (fun (_,_,x) -> 
        print_as_role x;
        print_newline ()) 
      desc ;
  debug "Parsing succeded";
  debug "Conversion to internal representation";
  let s = to_role_type desc in
  if Common.debug_p then debug (sprint_session s) ;
  debug "Generation of the code";
  let lg,(g,ra) = Generation.generate_code s in
  debug (Printf.sprintf "Creating module `%s.ml'" session_name);
  let f = open_out (session_name^".ml") in
  List.iter (output_string f) lg ;
  let _ = close_out f in
  debug (Printf.sprintf "Creating dot file `%s.dot'" session_name);
  Dotgen.gen_dot (session_name^".dot") g ra;
  

  debug "Generation of the interface";
  let lgi = Generation.generate_interface s in
  let fi = open_out (session_name^".mli") in
  debug (Printf.sprintf "Created interface `%s.mli'" session_name);
  List.iter (output_string fi) lgi ;
  let _ = close_out fi in
  debug "Compilation ending"
    


let rec parse_until_rang accum = 
   try 
     let s = input_char stdin in
     if s = '>' then 
       let sp = input_char stdin in
       if sp = '>' then 
         accum
       else
         (parse_until_rang  (accum^">"^(String.make 1 sp)))
     else 
       (parse_until_rang  (accum^(String.make 1 s)))
   with End_of_file -> accum

;;
    
(*let _ = parse_session ()*)
let _ = try 
    while(true) do
      let s = input_char stdin in
      if s = '<' then 
        let sp = input_char stdin in
        if sp = '<' then 
            let session = parse_until_rang "" in
            compile_session ("<<"^session^">>")
       (* else 
          begin
            print_char '<';print_char sp
          end)
      else
        print_char s*)
    done;
  with End_of_file -> exit 0 
    
