%{
  (********************************************************************)
  (* Session - Implementation                                         *)
  (*                                                                  *)
  (* parser.mly                                                       *)
  (********************************************************************)
  (* $Id: parser.mly 2847 2007-06-25 14:40:15Z denielou $ *)

  open Common
  open Syntax

  let debug = Common.debug Common.debug_p "parser"
    
  let error t info =
    debug ("Error generation: "^t) ;
    let (l,c1),(_,c2) = info in
    let s = Printf.sprintf "%d:%d-%d" l c1 c2 in
    if t = "" then raise (Common.Parse_error (s,info))
    else raise (Common.Parse_error ((*s^ ": " ^ *)t,info))


%}

%token <Common.info> ROLE SESSION
%token <Common.info> EOF
%token <Common.info> SEND RECE OR
%token <Common.info> LPA RPA
%token <Common.info> LANG
%token <Common.info * string> RANG
%token <Common.info> DOT MU SEMI EQUAL
%token <Common.info * string> IDENT
%token <Common.info * string> TYP
%start sess
%type <Syntax.ast> sess

%%

sess:
| LANG SESSION IDENT EQUAL desc RANG       { (snd $3, $5, snd $6) }
| LANG SESSION IDENT EQUAL RANG            {  error "Missing role declaration" ($4) }
| LANG SESSION IDENT RANG                  {  error "Missing equal" (fst $3) }
| LANG SESSION RANG                        {  error "Missing identifier" ($2) }
| LANG RANG                                {  error "Missing session" ($1) }


desc:
| ROLE IDENT TYP EQUAL session desc  { (snd $2,snd $3,$5)::$6 }
| ROLE IDENT TYP EQUAL session       { [snd $2,snd $3,$5] }
| ROLE IDENT TYP EQUAL               { error "Missing session description" ($4)}
| ROLE IDENT TYP                     { error "Missing equal" (fst $3)}
| ROLE IDENT                         { error "Missing type annotation" (fst $2)}
| ROLE                               { error "Missing role name" $1}
;

session:
| MU IDENT DOT session    { ASMu (merge_info $1 $3,snd $2,$4) }
| MU IDENT DOT            { error "Missing session description" $3 }
| MU IDENT                { ASMu (merge_info $1 (fst $2),snd $2,ASEnd)}
//| MU IDENT                { error "Missing dot" (fst $2) }
| MU                      { error "Missing label" $1 }
| LPA session RPA         { $2 }
| LPA                     { error "Missing right parenthesis" $1 }
| SEND msg                { ASSend ($1,$2) }
| SEND                    { error "Missing message declaration" $1 }
| RECE msg                { ASReceive ($1,$2) }
| RECE                    { error "Missing message declaration" $1 }
| IDENT                   { ASGoto (fst $1,snd $1) }
;

msg:
| IDENT TYP SEMI session                 { [(snd $1,snd $2),$4]}
| IDENT TYP SEMI                         { [(snd $1,snd $2),ASEnd]}
| IDENT TYP                              { [(snd $1,snd $2),ASEnd]}
| IDENT                                  { error "Missing colon and type information" (fst $1)}
| LPA msg OR listmsg RPA                 { $2@$4 }
| LPA msg RPA                            { $2 }
| LPA                                    { error "Missing message declaration" $1}
;

listmsg:
| IDENT TYP SEMI session OR listmsg { ((snd $1,snd $2),$4)::$6 }
| IDENT TYP SEMI session            { [(snd $1,snd $2),$4] }
| IDENT TYP SEMI                    { [(snd $1,snd $2),ASEnd] }
| IDENT TYP OR listmsg              { ((snd $1,snd $2),ASEnd)::$4 }
| IDENT TYP                         { [(snd $1,snd $2),ASEnd] }
| IDENT                             { error "Missing type information" (fst $1)}
|                                   { error "Wrong message declaration" (Common.bogusInfo) }
;


// Old stuff

/*

msgold:
| IDENT TYP SEMI session                 { [(snd $1,snd $2),$4]}
| IDENT TYP                              { [(snd $1,snd $2),ASEnd]}
| IDENT                                  { error "Missing type information" (fst $1)}
| LPA IDENT TYP SEMI session listmsg RPA { ((snd $2,snd $3),$5)::$6}
| LPA IDENT TYP SEMI session RPA         { [(snd $2,snd $3),$5] }
| LPA IDENT TYP SEMI                     { error "Missing right parenthesis" $1 }
| LPA IDENT TYP listmsg RPA              { ((snd $2,snd $3),ASEnd)::$4}
| LPA IDENT                              { error "Missing type information" (fst $2)}
| LPA                                    { error "Missing message declaration" $1}
;

*/
